@@ -1,5 +1,5 @@
 package Git::Raw::Diff;
-$Git::Raw::Diff::VERSION = '0.49';
+$Git::Raw::Diff::VERSION = '0.50';
 use strict;
 use warnings;
 
@@ -11,7 +11,7 @@ Git::Raw::Diff - Git diff class
 
 =head1 VERSION
 
-version 0.49
+version 0.50
 
 =head1 DESCRIPTION
 
@@ -132,9 +132,9 @@ Return a list of L<Git::Raw::Patch> objects for the diff.
 Generate text output from the diff object. The C<$callback> will be called for
 each line of the diff with two arguments: the first one represents the type of
 the patch line (C<"ctx"> for context lines, C<"add"> for additions, C<"del">
-for deletions, C<"file"> for file headers, C<"hunk"> for hunk headers or
-C<"bin"> for binary data) and the second argument contains the content of the
-patch line.
+for deletions, C<"file"> for file headers, C<"hunk"> for hunk headers, C<"bin">
+for binary data or C<"noeol"> if both files have no LF at end) and the second
+argument contains the content of the patch line.
 
 The C<$format> can be one of the following:
 
